/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Shock Detection on Accel Board outputs
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RD0 pin.
 * Test configuration:
 
     MCU:                     ac:ATmega16
                              http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf

     Dev.Board:               ac:EasyAVR6
                              http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/

     Oscillator:              External Clock 08.0000 MHz

     Ext. Modules:            ac:Accel_board (3-Axis)
                              http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                      mikroC PRO for AVR
                              http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RA0, Y-out to RA1 and Z-out to RA2.
     - PORTA LEDs and pull-up/down switches = Off
*/


// LCD module connections
sbit LCD_RS at PORTD2_bit;
sbit LCD_EN at PORTD3_bit;
sbit LCD_D4 at PORTD4_bit;
sbit LCD_D5 at PORTD5_bit;
sbit LCD_D6 at PORTD6_bit;
sbit LCD_D7 at PORTD7_bit;

sbit LCD_RS_Direction at DDD2_bit;
sbit LCD_EN_Direction at DDD3_bit;
sbit LCD_D4_Direction at DDD4_bit;
sbit LCD_D5_Direction at DDD5_bit;
sbit LCD_D6_Direction at DDD6_bit;
sbit LCD_D7_Direction at DDD7_bit;
// End LCD module connections

const signed int THRESHOLD = 40;        //  Choose the threshold value

signed int meas_x, meas_y, meas_z, old_x, old_y, old_z;
char shockedX = 0 , shockedY = 0, shockedZ = 0;

void DoMeasureXYZ() {
  meas_x = Adc_Read(0);   // Measure X_out
  meas_y = Adc_Read(1);   // Measure Y_out
  meas_z = Adc_Read(2);   // Measure Z_out
}

void main() {

  DDRA = 0xF8;                     // configure pins as input

  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  LCD_Out(1,1,"Monitoring...");

  DoMeasureXYZ();     // update values
  old_x = meas_x;
  old_y = meas_y;
  old_z = meas_z;
  //--- main loop
  while(1) {
    DoMeasureXYZ();   // measure new values

    if (abs(meas_x-old_x) > THRESHOLD)
      shockedX = 1 ;

    if (abs(meas_y-old_y) > THRESHOLD)
      shockedY = 1 ;

    if (abs(meas_z-old_z) > THRESHOLD)
      shockedZ = 1 ;

    if ( shockedX || shockedY || shockedZ)
      Lcd_Cmd(_LCD_CLEAR);

    if (shockedX)
      LCD_Out(1,1,"Shock... X axis");

    if (shockedY)
      LCD_Out(1,1,"Shock... Y axis");

    if (shockedZ)
      LCD_Out(1,1,"Shock... Z axis");

    if (shockedX || shockedY || shockedZ) {
      shockedX  = 0;
      shockedY  = 0;
      shockedZ  = 0;
      Delay_ms(1000);
      Lcd_Cmd(_LCD_CLEAR);
      LCD_Out(1,1,"Monitoring...");

      DoMeasureXYZ();    // update values
    }

    old_x = meas_x;
    old_y = meas_y;
    old_z = meas_z;
  }
}