/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Calibration and Reading Angles with Accel Board
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RD0 pin.
 * Test configuration:
 
     MCU:                     ac:ATmega16
                              http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf

     Dev.Board:               ac:EasyAVR6
                              http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/

     Oscillator:              External Clock 08.0000 MHz

     Ext. Modules:            ac:Accel_board (3-Axis)
                              http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                      mikroC PRO for AVR
                              http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RA0, Y-out to RA1 and Z-out to RA2.
     - PORTA LEDs and pull-up/down switches = Off
     - pull-up PORTB and set the buttons jumper to GND
     - Turn On RS-232 SW9.2 and 9.5
*/


// LCD module connections
sbit LCD_RS at PORTD2_bit;
sbit LCD_EN at PORTD3_bit;
sbit LCD_D4 at PORTD4_bit;
sbit LCD_D5 at PORTD5_bit;
sbit LCD_D6 at PORTD6_bit;
sbit LCD_D7 at PORTD7_bit;

sbit LCD_RS_Direction at DDD2_bit;
sbit LCD_EN_Direction at DDD3_bit;
sbit LCD_D4_Direction at DDD4_bit;
sbit LCD_D5_Direction at DDD5_bit;
sbit LCD_D6_Direction at DDD6_bit;
sbit LCD_D7_Direction at DDD7_bit;
// End LCD module connections

// --- Messages---
const char *msg1="Board to pos    ";
const char *msg2="and press RB0   ";
const char *msg3="Zero G values : ";
const char *msg4="One G values :  ";
const char *msg5="Angles:";
const char *msg6="X=     Y=     Z=    ";

char positionNo=1;

signed int xx, yy, zz;
signed int zeroG_x, zeroG_y, zeroG_z;
signed int oneG_x, oneG_y, oneG_z;
signed int meas_x, meas_y, meas_z;
char text[32];

void Usart_Write_Text(char* wr_txt) {
  while (*wr_txt)
    Uart1_Write(*wr_txt++);

  Uart1_Write(10);
  Uart1_Write(13);
}

// --- Copying strings from ROM to RAM
void strConstCpy(char *dest, const char *source) {
  while(*source)
  *dest++ = *source++ ;

  *dest = 0 ;
}

void DoMeasureXYZ() {
  meas_x = Adc_Read(0);   // Measure X_out
  meas_y = Adc_Read(1);   // Measure Y_out
  meas_z = Adc_Read(2);   // Measure Z_out
}

void WriteXYZ(signed int wr_x, signed int wr_y, signed int wr_z) {

  strConstCpy(text,msg6);

  if (wr_x < 0) {
     text[2] = '-';
     wr_x = -wr_x;
     }
  else
     text[2] = wr_x/1000+48;

  text[3] = (wr_x/100)%10+48;
  text[4] = (wr_x/10)%10+48;
  text[5] = wr_x%10+48;


  if (wr_y < 0) {
     text[9] = '-';
     wr_y = -wr_y;
     }
  else
     text[9] = wr_y/1000+48;

  text[10] = (wr_y/100)%10+48;
  text[11] = (wr_y/10)%10+48;
  text[12] = wr_y%10+48;


  if (wr_z < 0) {
     text[16] = '-';
     wr_z = -wr_z;
     }
  else
     text[16] = wr_z/1000+48;

  text[17] = (wr_z/100)%10+48;
  text[18] = (wr_z/10)%10+48;
  text[19] = wr_z%10+48;

  Usart_Write_Text(text);
}

void Calibrate() {

  strConstCpy(text,msg1);         //  1__ Put the Accel board in the position 1 :
  text[13] = positionNo++ + 48;   //      parallel to earth's surface
  Usart_Write_Text(text);         //      Z_axis is vertical with Z label UP
  strConstCpy(text,msg2);         //      to measure the Zero Gravity values for X and Y
  Usart_Write_Text(text);         //      and 1G Z value
  while (PINB.B0 == 1);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  zeroG_x = meas_x;
  zeroG_y = meas_y;
  oneG_z  = meas_z;
  Delay_ms(1000);


  strConstCpy(text,msg1);         //  2__ Put the Accel board in the position 2 :
  text[13] = positionNo++ + 48;   //      X_axis is vertical with X label UP
  Usart_Write_Text(text);
  strConstCpy(text,msg2);         //      to measure the 1G X value
  Usart_Write_Text(text);         //      and Zero Gravity value for Z
  while (PINB.B0 == 1);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  oneG_x = meas_x;
  zeroG_z = meas_z;
  Delay_ms(1000);

  strConstCpy(text,msg1);         //  3__ Put the Accel board in the position 3 :
  text[13] = positionNo++ + 48;   //      Y_axis is vertical with Y label UP
  Usart_Write_Text(text);
  strConstCpy(text,msg2);         //      to measure the 1G Y value
  Usart_Write_Text(text);
  while (PINB.B0 == 1);          //      Loop until RD0 becomes 1

  DoMeasureXYZ();
  oneG_y = meas_y;

  strConstCpy(text,msg3);         // Display ZeroG values
  Usart_Write_Text(text);
  WriteXYZ(zeroG_x, zeroG_y, zeroG_z);
  Delay_ms(3000);

  strConstCpy(text,msg4);         // Display OneG values
  Usart_Write_Text(text);
  WriteXYZ(oneG_x, oneG_y, oneG_z);
  Delay_ms(3000);
}

void Read_Angles() {
 double div_x, x_rad, div_y, y_rad, div_z, z_rad ;

  DoMeasureXYZ();
  div_x = (float)(meas_x-zeroG_x) / (oneG_x-zeroG_x);  // measured_G / one_G
  x_rad = asin( div_x );                               // x angle in radians
  xx = (x_rad*180) / 3.14;                              // x angle in degrees

  div_y = (float)(meas_y-zeroG_y) / (oneG_y-zeroG_y);  // measured_G / one_G
  y_rad = asin( div_y );                               // y angle in radians
  yy = (y_rad*180) / 3.14;                              // y angle in degrees

  div_z = (float)(meas_z-zeroG_z) / (oneG_z-zeroG_z);  // measured_G / one_G
  z_rad = asin( div_z );                               // z angle in radians
  zz = (z_rad*180) / 3.14;                             // z angle in degrees

  WriteXYZ(xx, yy, zz);
}

void main() {

  DDRA  = 0xF8;
  DDRD = 0;

  Uart1_Init(9600);   // Initialize USART module
  Delay_ms(100);

  Calibrate();

  strConstCpy(text,msg5);     // Display Angles
  Usart_Write_Text(text);

  //--- main loop
  while(1) {
    Read_Angles();
    Delay_ms(1000);
  }
}