/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Shock Detection on Accel Board outputs
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RA11 pin.
 * Test configuration:
 
     MCU:                     ac:dsPIC30F4013
                              http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf

     Dev.Board:               ac:EasydsPIC6
                              http://www.mikroe.com/eng/products/view/434/easydspic6-development-system/

     Oscillator:              XT with 8x PLL, 10.0000 MHz

     Ext. Modules:            ac:Accel_board (3-Axis)
                              http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                      mikroC PRO for dsPIC
                              http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RB5, Y-out to RB6 and Z-out to RB7.
     - PORTB LEDs and pull-up/down switches = Off
*/


// LCD module connections
sbit LCD_RS at LATD0_bit;
sbit LCD_EN at LATD1_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISD0_bit;
sbit LCD_EN_Direction at TRISD1_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

const signed int THRESHOLD = 40;        //  Choose the threshold value

signed int meas_x, meas_y, meas_z, old_x, old_y, old_z;
char shockedX = 0 , shockedY = 0, shockedZ = 0;

void DoMeasureXYZ() {
  meas_x = Adc1_Read(5);   // Measure X_out
  meas_y = Adc1_Read(6);   // Measure Y_out
  meas_z = Adc1_Read(7);   // Measure Z_out
}

void main() {
  ADPCFG = 0xFF0F;
  
  TRISB  = 0x00E0;                   // configure pins as input

  Lcd_Init();
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  LCD_Out(1,1,"Monitoring...");

  DoMeasureXYZ();     // update values
  old_x = meas_x;
  old_y = meas_y;
  old_z = meas_z;
  //--- main loop
  while(1) {
    DoMeasureXYZ();   // measure new values

    if (abs(meas_x-old_x) > THRESHOLD)
      shockedX = 1 ;

    if (abs(meas_y-old_y) > THRESHOLD)
      shockedY = 1 ;

    if (abs(meas_z-old_z) > THRESHOLD)
      shockedZ = 1 ;

    if ( shockedX || shockedY || shockedZ)
      Lcd_Cmd(_LCD_CLEAR);

    if (shockedX)
      LCD_Out(1,1,"Shock... X axis");

    if (shockedY)
      LCD_Out(1,1,"Shock... Y axis");

    if (shockedZ)
      LCD_Out(1,1,"Shock... Z axis");

    if (shockedX || shockedY || shockedZ) {
      shockedX  = 0;
      shockedY  = 0;
      shockedZ  = 0;
      Delay_ms(1000);
      Lcd_Cmd(_LCD_CLEAR);
      LCD_Out(1,1,"Monitoring...");

      DoMeasureXYZ();    // update values
    }

    old_x = meas_x;
    old_y = meas_y;
    old_z = meas_z;
  }
}