/*
 * Project name:
     3-Axis Accel Board
 * Copyright:
     (c) MikroElektronika, 2010.
 * Revision History:
     20101122:
       - initial release;
       - 20101129: (RR);
 * Description:
     This project demonstrates Shock Detection on Accel Board outputs
     using the ADConversion library.
     On transition to the next stage program waits for user to bring
     the Vcc to RA11 pin.
 * Test configuration:
 
     MCU:                     ac:dsPIC30F4013
                              http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf

     Dev.Board:               ac:EasydsPIC6
                              http://www.mikroe.com/eng/products/view/434/easydspic6-development-system/

     Oscillator:              XT with 8x PLL, 10.0000 MHz

     Ext. Modules:            ac:Accel_board (3-Axis)
                              http://www.mikroe.com/eng/products/view/133/three-axis-accelerometer-board/

     SW:                      mikroC PRO for dsPIC
                              http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/

 * NOTES:
     - When connecting the Vcc and Gnd consult the board schematics -
       if you don't do this properly you may DAMAGE the Accel board !!!
     - Connect the X-out to RB5, Y-out to RB6 and Z-out to RB7.
     - PORTA LEDs and pull-up/down switches = Off
     - PORTF LEDs On (SW12.4)
*/


const signed int THRESHOLD = 40;        //  Choose the threshold value

signed int meas_x, meas_y, meas_z, old_x, old_y, old_z;
char shockedX = 0 , shockedY = 0, shockedZ = 0;

void DoMeasureXYZ() {
  meas_x = Adc1_Read(5);   // Measure X_out
  meas_y = Adc1_Read(6);   // Measure Y_out
  meas_z = Adc1_Read(7);   // Measure Z_out
}

void main() {
  shockedX = 0;
  shockedY = 0;
  shockedZ = 0;

  ADPCFG = 0xFF0F;
  
  TRISB  = 0x00E0;                   // configure pins
  TRISF = 0x0000;
  
  PORTF = 0x0000;

  DoMeasureXYZ();     // update values
  old_x = meas_x;
  old_y = meas_y;
  old_z = meas_z;
  //--- main loop
  while(1) {
    DoMeasureXYZ();   // measure new values

    if (abs(meas_x-old_x) > THRESHOLD) {
      shockedX = 1;
      PORTF = 1;
    }
    if (abs(meas_y-old_y) > THRESHOLD) {
      shockedY = 1;
      PORTF = 2;
    }
    if (abs(meas_z-old_z) > THRESHOLD) {
      shockedZ = 1;
      PORTF = 4;
    }

    if (shockedX || shockedY || shockedZ) {
      shockedX  = 0;
      shockedY  = 0;
      shockedZ  = 0;
      Delay_ms(1000);
      PORTF = 0;

      DoMeasureXYZ();    // update values
    }

    old_x = meas_x;
    old_y = meas_y;
    old_z = meas_z;
  }
}